;==============================================================================
;    clock.asm - RTC support using a 32.768kHz crystal on timer 1
;==============================================================================

;------------------------------------------------------------------------------
;
; Server monitor firmware - clock routines
; File Version 1 - 31 Oct 10
; hairymnstr@gmail.com
;
; Copyright (C) 2010  Nathan Dumont
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;------------------------------------------------------------------------------

list p=18f4520
include <p18f4520.inc>

; -- Externals From screen.asm ------------------------------------------------
    extern      screen_set_row
    extern      screen_print
    extern      screen_bl_on
    extern      screen_bl_off

  UDATA

clock_secs      RES     1
clock_mins      RES     1
clock_hours     RES     1
clock_days      RES     1
clock_month     RES     1
clock_year      RES     1
  
  CODE

clock_init
    movlw       0x80
    movwf       TMR1H           ; pre-load to overflow after 32k not 64k
    clrf        TMR1L           ; i.e. 1S interrupt
    movlw       0x0F
    movwf       T1CON           ; setup for async external clock source
    clrf        clock_secs
    clrf        clock_mins
    clrf        clock_hours
    clrf        clock_days
    clrf        clock_month
    clrf        clock_year

    bcf         PIR1, TMR1IF
    bsf         PIE1, TMR1IE
    bsf         INTCON, GIE     ; enable 1 second interrupt

    return

clock_int
    btfsc       TMR1L, 0
    bra         clock_int
clock_int1
    btfss       TMR1L, 0
    bra         clock_int1

    bcf         INTCON, GIE     ; disable interrupts until this is done, don't want serial updating time while we are.
    bsf         TMR1H, 7
    bcf         PIR1, TMR1IF    ; clear interrupt flag

    call        clock_int_update
    bsf         INTCON, GIE
    call        clock_print_time
    return

clock_int_update
    incf        clock_secs, w
    daw
    movwf       clock_secs
    movlw       0x59
    cpfsgt      clock_secs
    return
    ; seconds register overflowed
    clrf        clock_secs      ; zero this
    incf        clock_mins, w
    daw
    movwf       clock_mins
    movlw       0x59
    cpfsgt      clock_mins
    return
    ; minutes overflowed
    clrf        clock_mins
    incf        clock_hours, w
    daw
    movwf       clock_hours
    movlw       0x23
    cpfsgt      clock_hours
    return
    ; hours overflowed
    clrf        clock_hours
    ; days etc. not implemented yet
    return

; return an ascii representation of the lower nibble of a BCD variable
clock_bcdasc
    andlw       0x0F
    addlw       '0'
    return
    
clock_print_time
    movlw       0x07
    call        screen_set_row

    swapf       clock_hours,w
    call        clock_bcdasc
    call        screen_print
    movf        clock_hours,w
    call        clock_bcdasc
    call        screen_print

    movlw       ':'
    call        screen_print

    swapf       clock_mins,w
    call        clock_bcdasc
    call        screen_print
    movf        clock_mins,w
    call        clock_bcdasc
    call        screen_print
    
    movlw       ':'
    call        screen_print

    swapf       clock_secs,w
    call        clock_bcdasc
    call        screen_print
    movf        clock_secs,w
    call        clock_bcdasc
    call        screen_print

    return

; -- Export These --------------------------------------------------------------

    GLOBAL      clock_init
    GLOBAL      clock_int
    GLOBAL      clock_secs      ; export these so that the serial link can set time.
    GLOBAL      clock_mins
    GLOBAL      clock_hours
    GLOBAL      clock_days
    GLOBAL      clock_month
    GLOBAL      clock_year

    END