;==============================================================================
;    main.asm
;==============================================================================

;------------------------------------------------------------------------------
;
; Server monitor firmware - main routine and bootloader
; File Version 1 - 21 Apr 10
; File Version 2 - 31 Oct 10
; hairymnstr@gmail.com
;
; Copyright (C) 2010  Nathan Dumont
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;------------------------------------------------------------------------------

list p=18f4520
include p18f4520.inc

errorlevel -302
errorlevel -205

CONFIG OSC = HSPLL
CONFIG FCMEN = ON, IESO = OFF, PWRT = ON, BOREN = OFF, BORV = 0
CONFIG WDT = OFF, WDTPS = 1, MCLRE = ON, LPT1OSC = OFF
CONFIG PBADEN = OFF, CCP2MX = PORTC, STVREN = ON, LVP = OFF
CONFIG XINST = OFF, DEBUG = OFF, CP0 = OFF, CP1 = OFF, CP2 = OFF, CP3 = OFF
CONFIG CPB = OFF, CPD = OFF, WRT0 = OFF, WRT1 = OFF, WRT2 = OFF, WRT3 = OFF
CONFIG WRTB = OFF, WRTC = OFF, WRTD = OFF, EBTR0 = OFF, EBTR1 = OFF
CONFIG EBTR2 = OFF, EBTR3 = OFF, EBTRB = OFF

; -- Externals from screen.asm -------------------------------------------------

    extern      screen_restart
    extern      screen_bl_on
    extern      screen_set_row
    extern      screen_print
    extern      screen_clear

; -- Externals from serial.asm -------------------------------------------------
    extern      serial_init
    extern      serial_rx_int

; -- Externals from clock.asm --------------------------------------------------
    extern      clock_init
    extern      clock_int

    UDATA

COUNT   RES     1
boot_count      RES     1
boot_flags      RES     1
boot_first_pass EQU     0
boot_check      RES     1
boot_addrl      RES     1
boot_addrh      RES     1
boot_addru      RES     1


    org 0x00
start
    goto        main

    org 0x08
    goto        interrupt

    org 0x20
boot_loader_entry
    bcf         INTCON, GIE             ; no interrupts while in bootloader mode
boot_loader
    movlw       d'68'
    movwf       boot_count
    clrf        boot_check
    movlw       0x01
    movwf       FSR0H
    clrf        FSR0L
    bcf         LATC,5

boot_read_loop
    ; wait for a byte
    btfss       PIR1, RCIF
    bra         boot_read_loop
    movf        RCREG, w
    movwf       POSTINC0
    xorwf       boot_check, f
    decfsz      boot_count,f
    bra         boot_read_loop

    bsf         LATC, 5  
; got a whole 68 byte packet, check that checksum is okay
    movlw       0x00
    xorwf       boot_check, w
    bnz         boot_nak

; now see what type of packet we have (based on Intel Hex file format)
    clrf        FSR0L
; data type 00
    movlw       0x00
    xorwf       INDF0, w
    btfsc       STATUS, Z
    goto        boot_data_packet

; end of file type 01
    movlw       0x01
    xorwf       INDF0, w
    btfsc       STATUS, Z
    goto        boot_end

; extended linear address type 04
    movlw       0x04
    xorwf       INDF0, w
    btfsc       STATUS, Z
    goto        boot_elar
    
; otherwise we don't know what this packet is so return a nak
    goto        boot_nak

; boot_data_packet - the 68 byte message contains the low 16 bits of the
; address in flash and the 64 bytes of data to be stored

boot_data_packet
    incf        FSR0L,f         ; move onto first address byte
    movf        POSTINC0, w
    movwf       boot_addrh
    movf        POSTINC0, w
    movwf       boot_addrl
    movlw       LOW bootloader_ends
    subwf       boot_addrl, w           ; set carry flag for low byte
    movlw       HIGH bootloader_ends    ; get the high byte
    subwfb      boot_addrh, w           ; subtract the high address
    ; if carry is clear (i.e. borrow is true) now, the address specified
    ; is within the bootloader and will not be written to for safety
    btfsc       STATUS, C
    bra         boot_data_addr_ok
    ; there is one exception, if the upper address is set then the address
    ; is accepted
    movlw       0x00
    xorwf       boot_addru,w
    bnz         boot_data_addr_ok
    
    ; definitely trying to overwrite bootloader, send a nak
    goto        boot_nak

boot_data_addr_ok
    movf        boot_addrl,w
    movwf       TBLPTRL
    movf        boot_addrh,w
    movwf       TBLPTRH
    movf        boot_addru,w
    movwf       TBLPTRU

    ; now erase the 64K block we're in
    bsf         EECON1, EEPGD
    bcf         EECON1, CFGS
    bsf         EECON1, WREN
    bsf         EECON1, FREE
    
    movlw       0x55
    movwf       EECON2
    movlw       0xAA
    movwf       EECON2
    bsf         EECON1, WR

    ; 64 bytes erased.  Next do a dummy decrement because we're going
    ; to use pre-inc so that we're still in the right block after writing
    ; to call write
    tblrd*-
    bsf         boot_flags, boot_first_pass          ; two 32 byte writes so need to keep track
boot_data_write
    movlw       0x20
    movwf       boot_count
boot_data_write_loop
    movf        POSTINC0, w
    movwf       TABLAT
    tblwt+*
    decfsz      boot_count,f
    bra         boot_data_write_loop

    ; written 32 bytes, call eewrite to flush to the flash
    bsf         EECON1, EEPGD
    bcf         EECON1, CFGS
    bsf         EECON1, WREN
    
    movlw       0x55
    movwf       EECON2
    movlw       0xAA
    movwf       EECON2
    bsf         EECON1, WR

    bcf         EECON1, WREN

    btfss       boot_flags, boot_first_pass
    bra         boot_data_validate
    bcf         boot_flags, boot_first_pass          ; not any more
    bra         boot_data_write                 ; second chunk of 32 bytes

boot_data_validate
    movlw       0x03
    movwf       FSR0L                           ; point to first data byte in packet

    movf        boot_addrl, w
    movwf       TBLPTRL
    movf        boot_addrh, w
    movwf       TBLPTRH
    movf        boot_addru, w
    movwf       TBLPTRU

    movlw       0x40
    movwf       boot_count                      ; number of bytes to check == d'64'

boot_data_validate_loop
    movf        POSTINC0, w
    tblrd*+
    xorwf       TABLAT, w
    btfss       STATUS, Z
    goto        boot_nak                        ; it's not zero so there was an error; nak
    decfsz      boot_count, f
    bra         boot_data_validate_loop

    ; if we got here then all the data matched.
    goto        boot_ack

; boot_end - called to finish writing to flash.  This just replies with an ACK then
; does a full reset on the PIC
boot_end
    movlw       0x06
boot_end_wait1
    btfss       PIR1, TXIF
    bra         boot_end_wait1
    movwf       TXREG
; now need to wait for the shift register to be completely empty before doing the reset
boot_end_wait2
    btfss       TXSTA, TRMT
    bra         boot_end_wait2

    reset

; boot_elar - called to set the extended address record
boot_elar
    ; ignore the first byte
    movf        POSTINC0,w
    ; save the second byte as the upper address
    movf        POSTINC0,w
    movwf       boot_addru
    ; send an ack
    goto        boot_ack

boot_ack
    movlw       0x06
boot_ack_loop
    btfss       PIR1, TXIF
    bra         boot_ack_loop
    movwf       TXREG       
    goto        boot_loader

boot_nak
    movlw       0x15
boot_nak_loop
    btfss       PIR1, TXIF
    bra         boot_nak_loop
    movwf       TXREG
    goto        boot_loader

org     0x200           ; needs to be erase-page aligned
bootloader_ends

interrupt
    btfsc       PIR1, TMR1IF
    call        clock_int
    btfsc       PIR1, RCIF
    call        serial_rx_int
    retfie      FAST

org     0x300
main
    movlw       0xF0
    movwf       TRISB
    clrf        LATB

    movlw       0x0F
    movwf       ADCON1

    call        serial_init

    call        screen_restart

    call        screen_clear

    call        clock_init

    movlw       0x03
    call        screen_set_row

;     movlw       'H'
;     call        screen_print_at
;     movlw       'i'
;     call        screen_print_at

    clrf        COUNT

splash_loop
    movlw       LOW boot_msg
    addwf       COUNT, w
    movwf       TBLPTRL
    movlw       0x00
    btfsc       STATUS, C
    incf        WREG, f
    addlw       HIGH boot_msg
    movwf       TBLPTRH
    clrf        TBLPTRU
    incf        COUNT, f
    movlw       0x00
    tblrd*+
    xorwf       TABLAT,w
    bz          splash_done
    call        screen_print
    bra         splash_loop

splash_done

    call        screen_bl_on


loop
    clrf        COUNT
    clrf        COUNT+1
    movlw       0x20
    movwf       COUNT+2
    btg         LATB, 0
loop_inner
    decfsz      COUNT,f
    bra         loop_inner
    decfsz      COUNT+1,f
    bra         loop_inner
    decfsz      COUNT+2,f
    bra         loop_inner
    bra         loop


boot_msg
    data        0x2020
    data        0x7953, 0x7473, 0x6D65, 0x5320, 0x6174
    data        0x7472, 0x6E69, 0x2E67, 0x2E2E, 0x0000

; == Export These =============================================================

    GLOBAL      boot_loader_entry

    end
