;*******************************************************************************
;*  Screen2 - second edition of KS0108/0107 driver for PIC                     *
;*                                                                             *
;*  - Now operates on a set of lines.                                          *
;*  - No auto scroll features or ASCII control characters                      *
;*  - No overlay support, this is write only pixels are set only               *
;*                                                                             *
;*******************************************************************************

;------------------------------------------------------------------------------
;
; Server monitor firmware - screen driver routines
; File Version 1 - 31 Oct 10
; hairymnstr@gmail.com
;
; Copyright (C) 2010  Nathan Dumont
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;------------------------------------------------------------------------------

list p=18f4520
include p18f4520.inc
include screen_pins.inc

    UDATA

screen_col      RES     1
screen_row      RES     1
screen_count    RES     1
delay_reg       RES     3
screen_write_temp       RES     1

    CODE

;*******************************************************************************
;       START OF SCREEN FUNCTIONS
;*******************************************************************************

;*******************************************************************************
;*                                                                             *
;*      SCREEN_RESTART - resets the screen so it is ready for use              *
;*                       NOTE: doesn't clear screen, use CLS                   *
;*                                                                             *
;*      Functions called:       DELAY, S_CLK, S_LD                             *
;*      Registers affected:     none                                           *
;*                                                                             *
;*******************************************************************************

screen_restart
    ; make sure all the pins are set to the correct I/O
    bcf         SCREEN_RST, SCREEN_RST_PIN      ;make sure that RST is low    
    bcf         SCREEN_RST_DIR, SCREEN_RST_PIN
    bsf         SCREEN_CS1, SCREEN_CS1_PIN
    bcf         SCREEN_CS1_DIR, SCREEN_CS1_PIN
    bsf         SCREEN_CS2, SCREEN_CS2_PIN
    bcf         SCREEN_CS2_DIR, SCREEN_CS2_PIN
    bcf         SCREEN_E, SCREEN_E_PIN
    bcf         SCREEN_E_DIR, SCREEN_E_PIN
    bsf         SCREEN_DI, SCREEN_DI_PIN
    bcf         SCREEN_DI_DIR, SCREEN_DI_PIN
    bsf         SCREEN_RW, SCREEN_RW_PIN
    bcf         SCREEN_RW_DIR, SCREEN_RW_PIN
    bcf         SCREEN_BL, SCREEN_BL_PIN
    bcf         SCREEN_BL_DIR, SCREEN_BL_PIN

    call        DELAY                           ;give the screen time to notice
    bsf         SCREEN_RST, SCREEN_RST_PIN      ;Screen RST high, starts up low
    call        DELAY                           ;give the screen time to notice again

    bcf         SCREEN_CS1, SCREEN_CS1_PIN      ;select chip1
    bcf         SCREEN_CS2, SCREEN_CS2_PIN      ;select chip2

    movlw       0x3F
    call        screen_load_inst

    movlw       0x40
    call        screen_load_inst

    movlw       0xB8
    call        screen_load_inst

    call        screen_clear
    return


screen_load_inst
    movwf       screen_write_temp
    bcf         SCREEN_DI, SCREEN_DI_PIN
    bsf         SCREEN_RW, SCREEN_RW_PIN
screen_load_inst_check_status
    call        screen_delay
    bsf         SCREEN_E, SCREEN_E_PIN
    call        screen_delay
    movf        SCREEN_PORT, w
    bcf         SCREEN_E, SCREEN_E_PIN
    btfsc       WREG, 7
    bra         screen_load_inst_check_status
    ; status is now "ready"
    bcf         SCREEN_RW, SCREEN_RW_PIN
    clrf        SCREEN_DIR
    movf        screen_write_temp, w
    movwf       SCREEN_LAT
    call        screen_delay
    bsf         SCREEN_E, SCREEN_E_PIN
    call        screen_delay
    bcf         SCREEN_E, SCREEN_E_PIN
    setf        SCREEN_DIR
    return

screen_load_data
    movwf       screen_write_temp
    bcf         SCREEN_DI, SCREEN_DI_PIN
    bsf         SCREEN_RW, SCREEN_RW_PIN
screen_load_data_check_status
    call        screen_delay
    bsf         SCREEN_E, SCREEN_E_PIN
    call        screen_delay
    movf        SCREEN_PORT, w
    bcf         SCREEN_E, SCREEN_E_PIN
    btfsc       WREG, 7
    bra         screen_load_data_check_status
    ; status is now "ready"
    bcf         SCREEN_RW, SCREEN_RW_PIN
    bsf         SCREEN_DI, SCREEN_DI_PIN
    clrf        SCREEN_DIR
    movf        screen_write_temp, w
    movwf       SCREEN_LAT
    call        screen_delay
    bsf         SCREEN_E, SCREEN_E_PIN
    call        screen_delay
    bcf         SCREEN_E, SCREEN_E_PIN
    setf        SCREEN_DIR
    return

;*******************************************************************************
;*                                                                             *
;*      CLS - Blank the screen and reset internal counters.                    *
;*                                                                             *
;*      Inputs:                         None                                   *
;*      Returns:                        None                                   *
;*      Calls:                          screen_newline                         *
;*      Affects:                        LATD, M_S_COL, M_S_PG                  *
;*                                                                             *
;*******************************************************************************

screen_clear
    clrf        screen_row
    clrf        screen_col

    bcf         SCREEN_CS1, SCREEN_CS1_PIN      ; select both halves of the
    bcf         SCREEN_CS2, SCREEN_CS2_PIN      ; screen to save time
screen_clear_line_loop
    movlw       0xB8
    iorwf       screen_row, w
    call        screen_load_inst                ; set the page (up-down)

    movlw       0x40
    call        screen_load_inst                ; set to the first column
    movlw       0x40
    movwf       screen_col                        ; count 64 pixels

screen_clear_col_loop
    movlw       0x00
    call        screen_load_data
    decfsz      screen_col, f
    bra         screen_clear_col_loop

    incf        screen_row, f                     ; increment the row
    btfss       screen_row, 3                     ; only 8 rows
    bra         screen_clear_line_loop

    clrf        screen_col
    clrf        screen_row
    return

;*******************************************************************************
;*                                                                             *
;*      screen_set_row - selects the active character row (0-7)                *
;*                                                                             *
;*      Inputs:         Row to select in W                                     *
;*      Returns:        None                                                   *
;*      Calls:          screen_load_inst                                       *
;*      Affects:        screen_row, screen_col (set to 0)                      *
;*                                                                             *
;*******************************************************************************

screen_set_row
    andlw       0x7
    movwf       screen_row
    clrf        screen_col

    bcf         SCREEN_CS1, SCREEN_CS1_PIN
    bcf         SCREEN_CS2, SCREEN_CS2_PIN
    iorlw       0xB8
    call        screen_load_inst
    movlw       0x40
    call        screen_load_inst

    bsf         SCREEN_CS1, SCREEN_CS1_PIN
    bsf         SCREEN_CS2, SCREEN_CS2_PIN

    return

;*******************************************************************************
;*                                                                             *
;*      screen_set_col - select the active character column (0-127)            *
;*                                                                             *
;*      Inputs:         Col to select in W                                     *
;*      Returns:        None                                                   *
;*      Calls:          screen_load_inst                                       *
;*      Affects:        screen_col                                             *
;*                                                                             *
;*******************************************************************************

screen_set_col
    andlw       0x7F
    movwf       screen_col

    bsf         SCREEN_CS1, SCREEN_CS1_PIN
    bsf         SCREEN_CS2, SCREEN_CS2_PIN
    btfsc       screen_col, 6
    bcf         SCREEN_CS2, SCREEN_CS2_PIN
    btfss       screen_col, 6
    bcf         SCREEN_CS1, SCREEN_CS1_PIN

    movlw       0x40
    iorwf       screen_col, w
    call        screen_load_inst

    bsf         SCREEN_CS1, SCREEN_CS1_PIN
    bsf         SCREEN_CS2, SCREEN_CS2_PIN

    return

;*******************************************************************************
;*                                                                             *
;*      screen_write_byte - ??                                 
;*                                                                             *
;*      Inputs:         The byte to write to screen in W                       *
;*      Returns:        zero flag set if okay, not set if no space left        *
;*      Calls:          screen_load_data                                       *
;*      Affects:        screen_col                                             *
;*                                                                             *
;*******************************************************************************

screen_write_byte
    bcf         STATUS, Z               ; prepare error response
    btfsc       screen_col, 7           ; check for column 128 (off screen)
    return                              ; return error if in col 128

    bsf         SCREEN_CS1, SCREEN_CS1_PIN
    bsf         SCREEN_CS2, SCREEN_CS2_PIN
    btfss       screen_col,6
    bcf         SCREEN_CS1, SCREEN_CS1_PIN      ; select CS1 if col < 64
    btfsc       screen_col,6
    bcf         SCREEN_CS2, SCREEN_CS2_PIN      ; select CS2 if col >= 64

    call        screen_load_data                ; write the byte to the screen

    incf        screen_col, f           ; increment the current column
                                        ; screen's internal counter autoincs
    xorwf       WREG,w                  ; will set the Z flag

    return

;*******************************************************************************
;*                                                                             *
;*      screen_print - print the ASCII character from W                        *
;*                                                                             *
;*      Inputs:         The ASCII character code to print                      *
;*      Returns:        None                                                   *
;*      Calls:          screen_write_byte                                      *
;*      Affects:        None                                                   *
;*                                                                             *
;*******************************************************************************

screen_print
    clrf        TBLPTRU
    clrf        TBLPTRH
    rlcf        WREG,f
    btfsc       STATUS,C
    bsf         TBLPTRH, 2
    bcf         STATUS, C
    rlcf        WREG,f
    btfsc       STATUS, C
    bsf         TBLPTRH, 1
    bcf         STATUS, C
    rlcf        WREG, f
    btfsc       STATUS, C
    bsf         TBLPTRH, 0

    addlw       LOW CHAR_TBL
    btfsc       STATUS, C
    incf        TBLPTRH,f
    movwf       TBLPTRL

    movlw       HIGH CHAR_TBL
    addwf       TBLPTRH,f

    movlw       0x06
    movwf       screen_count
screen_print_loop
    tblrd*+
    movf        TABLAT, w
    call        screen_write_byte
    decfsz      screen_count, f
    bra         screen_print_loop

    return


screen_bl_on
    bsf         SCREEN_BL, SCREEN_BL_PIN
    return

screen_bl_off
    bcf         SCREEN_BL, SCREEN_BL_PIN
    return

;===============================================================================
; screen_delay - short delay for screen clock ~ 600ns
;===============================================================================

screen_delay
    nop
    nop
    nop
    nop
    return


;*******************************************************************************
;       END OF SCREEN FUNCTIONS
;*******************************************************************************

;*******************************************************************************
;*                                                                             *
;*      DELAY - Long delay (~20ms)                                             *
;*                                                                             *
;*      Inputs:                         None                                   *
;*      Returns:                        None                                   *
;*      Calls:                          None                                   *
;*      Affects:                        M_DELAY                                *
;*                                                                             *
;*******************************************************************************

DELAY
    setf delay_reg
    setf delay_reg+1
DELAY_LOOP
    decfsz delay_reg,f
    goto DELAY_LOOP
    decfsz delay_reg+1,f
    goto DELAY_LOOP
    return

; == Bitmapped font stored below.  Doesn't need to be at a fixed location ======

    ORG 0x7800
CHAR_TBL
        data 0x0000, 0x0000, 0x0000, 0x0000 ;0
        data 0x0000, 0x0000, 0x0000, 0x0000 ;1
        data 0x0000, 0x0000, 0x0000, 0x0000 ;2
        data 0x0000, 0x0000, 0x0000, 0x0000 ;3
        data 0x0000, 0x0000, 0x0000, 0x0000 ;4
        data 0x0000, 0x0000, 0x0000, 0x0000 ;5
        data 0x0000, 0x0000, 0x0000, 0x0000 ;6
        data 0x0000, 0x0000, 0x0000, 0x0000 ;7
        data 0x0000, 0x0000, 0x0000, 0x0000 ;8
        data 0x0000, 0x0000, 0x0000, 0x0000 ;9
        data 0x0000, 0x0000, 0x0000, 0x0000 ;10
        data 0x0000, 0x0000, 0x0000, 0x0000 ;11
        data 0x0000, 0x0000, 0x0000, 0x0000 ;12
        data 0x0000, 0x0000, 0x0000, 0x0000 ;13
        data 0x0000, 0x0000, 0x0000, 0x0000 ;14
        data 0x0000, 0x0000, 0x0000, 0x0000 ;15
        data 0x0000, 0x0000, 0x0000, 0x0000 ;16
        data 0x0000, 0x0000, 0x0000, 0x0000 ;17
        data 0x0000, 0x0000, 0x0000, 0x0000 ;18
        data 0x0000, 0x0000, 0x0000, 0x0000 ;19
        data 0x0000, 0x0000, 0x0000, 0x0000 ;20
        data 0x0000, 0x0000, 0x0000, 0x0000 ;21
        data 0x0000, 0x0000, 0x0000, 0x0000 ;22
        data 0x0000, 0x0000, 0x0000, 0x0000 ;23
        data 0x0000, 0x0000, 0x0000, 0x0000 ;24
        data 0x0000, 0x0000, 0x0000, 0x0000 ;25
        data 0x0000, 0x0000, 0x0000, 0x0000 ;26
        data 0x0000, 0x0000, 0x0000, 0x0000 ;27
        data 0x0000, 0x0000, 0x0000, 0x0000 ;28
        data 0x0000, 0x0000, 0x0000, 0x0000 ;29
        data 0x0000, 0x0000, 0x0000, 0x0000 ;30
        data 0x0000, 0x0000, 0x0000, 0x0000 ;31
        data 0x0000, 0x0000, 0x0000, 0x0000 ;32  
        data 0x0000, 0x002F, 0x0000, 0x0000 ;33 !
        data 0x0300, 0x0300, 0x0000, 0x0000 ;34 "
        data 0x3E14, 0x3E14, 0x0014, 0x0000 ;35 #
        data 0x1600, 0x1A3F, 0x0000, 0x0000 ;36 $
        data 0x1562, 0x542A, 0x0023, 0x0000 ;37 %
        data 0x2916, 0x2816, 0x0000, 0x0000 ;38 &
        data 0x0000, 0x0003, 0x0000, 0x0000 ;39 '
        data 0x0000, 0x211E, 0x0000, 0x0000 ;40 (
        data 0x2100, 0x001E, 0x0000, 0x0000 ;41 )
        data 0x0500, 0x0502, 0x0000, 0x0000 ;42 *
        data 0x0808, 0x083E, 0x0008, 0x0000 ;43 +
        data 0x2000, 0x0010, 0x0000, 0x0000 ;44 ,
        data 0x0800, 0x0808, 0x0000, 0x0000 ;45 -
        data 0x0000, 0x0020, 0x0000, 0x0000 ;46 .
        data 0x0830, 0x0304, 0x0000, 0x0000 ;47 /
        data 0x291E, 0x2325, 0x001E, 0x0000 ;48 0
        data 0x2200, 0x203F, 0x0000, 0x0000 ;49 1
        data 0x3122, 0x2529, 0x0022, 0x0000 ;50 2
        data 0x2112, 0x2525, 0x001A, 0x0000 ;51 3
        data 0x1418, 0x3F12, 0x0010, 0x0000 ;52 4
        data 0x2517, 0x2525, 0x0019, 0x0000 ;53 5
        data 0x251E, 0x2525, 0x0019, 0x0000 ;54 6
        data 0x0101, 0x0539, 0x0003, 0x0000 ;55 7
        data 0x251A, 0x2525, 0x001A, 0x0000 ;56 8
        data 0x2512, 0x2525, 0x001E, 0x0000 ;57 9
        data 0x0000, 0x0012, 0x0000, 0x0000 ;58 :
        data 0x2000, 0x0012, 0x0000, 0x0000 ;59 ;
        data 0x0800, 0x2214, 0x0000, 0x0000 ;60 <
        data 0x1400, 0x1414, 0x0014, 0x0000 ;61 =
        data 0x2200, 0x0814, 0x0000, 0x0000 ;62 >
        data 0x0102, 0x0559, 0x0002, 0x0000 ;63 ?
        data 0x493E, 0x5D55, 0x000E, 0x0000 ;64 @
        data 0x0A3C, 0x0A09, 0x003C, 0x0000 ;65 A
        data 0x253F, 0x2525, 0x001A, 0x0000 ;66 B
        data 0x211E, 0x2121, 0x0012, 0x0000 ;67 C
        data 0x213F, 0x2121, 0x001E, 0x0000 ;68 D
        data 0x253F, 0x2125, 0x0021, 0x0000 ;69 E
        data 0x053F, 0x0105, 0x0001, 0x0000 ;70 F
        data 0x211E, 0x1929, 0x003A, 0x0000 ;71 G
        data 0x043F, 0x0404, 0x003F, 0x0000 ;72 H
        data 0x2100, 0x213F, 0x0000, 0x0000 ;73 I
        data 0x2010, 0x1F20, 0x0000, 0x0000 ;74 J
        data 0x043F, 0x110A, 0x0020, 0x0000 ;75 K
        data 0x203F, 0x2020, 0x0020, 0x0000 ;76 L
        data 0x023F, 0x0204, 0x003F, 0x0000 ;77 M
        data 0x023F, 0x0804, 0x003F, 0x0000 ;78 N
        data 0x211E, 0x2121, 0x001E, 0x0000 ;79 O
        data 0x093F, 0x0909, 0x0006, 0x0000 ;80 P
        data 0x211E, 0x1129, 0x002E, 0x0000 ;81 Q
        data 0x093F, 0x0909, 0x0036, 0x0000 ;82 R
        data 0x2522, 0x2525, 0x0019, 0x0000 ;83 S
        data 0x0101, 0x013F, 0x0001, 0x0000 ;84 T
        data 0x201F, 0x2020, 0x001F, 0x0000 ;85 U
        data 0x100F, 0x1020, 0x000F, 0x0000 ;86 V
        data 0x201F, 0x2010, 0x001F, 0x0000 ;87 W
        data 0x0A31, 0x0A04, 0x0031, 0x0000 ;88 X
        data 0x0201, 0x023C, 0x0001, 0x0000 ;89 Y
        data 0x2931, 0x2325, 0x0021, 0x0000 ;90 Z
        data 0x0000, 0x213F, 0x0000, 0x0000 ;91 [
        data 0x0403, 0x3008, 0x0000, 0x0000 ;92 \
        data 0x2100, 0x003F, 0x0000, 0x0000 ;93 ]
        data 0x0200, 0x0201, 0x0000, 0x0000 ;94 ^
        data 0x2020, 0x2020, 0x0020, 0x0000 ;95 _
        data 0x0100, 0x0002, 0x0000, 0x0000 ;96 `
        data 0x2A10, 0x3C2A, 0x0000, 0x0000 ;97 a
        data 0x243F, 0x1824, 0x0000, 0x0000 ;98 b
        data 0x2418, 0x2424, 0x0000, 0x0000 ;99 c
        data 0x2418, 0x3F24, 0x0000, 0x0000 ;100 d
        data 0x2A1C, 0x2C2A, 0x0000, 0x0000 ;101 e
        data 0x3C00, 0x020A, 0x0000, 0x0000 ;102 f
        data 0x4C00, 0x5252, 0x003C, 0x0000 ;103 g
        data 0x043F, 0x3804, 0x0000, 0x0000 ;104 h
        data 0x0000, 0x003A, 0x0000, 0x0000 ;105 i
        data 0x4000, 0x003A, 0x0000, 0x0000 ;106 j
        data 0x083F, 0x2214, 0x0000, 0x0000 ;107 k
        data 0x0000, 0x201F, 0x0000, 0x0000 ;108 l
        data 0x043C, 0x0438, 0x0038, 0x0000 ;109 m
        data 0x043C, 0x3804, 0x0000, 0x0000 ;110 n
        data 0x2418, 0x1824, 0x0000, 0x0000 ;111 o
        data 0x147C, 0x0814, 0x0000, 0x0000 ;112 p
        data 0x1408, 0x7C14, 0x0000, 0x0000 ;113 q
        data 0x043C, 0x0804, 0x0000, 0x0000 ;114 r
        data 0x2A24, 0x122A, 0x0000, 0x0000 ;115 s
        data 0x1E00, 0x2028, 0x0000, 0x0000 ;116 t
        data 0x201C, 0x3C20, 0x0000, 0x0000 ;117 u
        data 0x100C, 0x1020, 0x000C, 0x0000 ;118 v
        data 0x201C, 0x2010, 0x001C, 0x0000 ;119 w
        data 0x1422, 0x1408, 0x0022, 0x0000 ;120 x
        data 0x504C, 0x3C50, 0x0000, 0x0000 ;121 y
        data 0x3424, 0x242C, 0x0000, 0x0000 ;122 z
        data 0x0800, 0x4136, 0x0000, 0x0000 ;123 {
        data 0x0000, 0x003F, 0x0000, 0x0000 ;124 |
        data 0x4100, 0x0836, 0x0000, 0x0000 ;125 }
        data 0x0408, 0x1008, 0x0008, 0x0000 ;126 ~
        data 0x0000, 0x0000, 0x0000, 0x0000 ;127 DEL

    GLOBAL      screen_restart
    GLOBAL      screen_bl_on
    GLOBAL      screen_bl_off
    GLOBAL      screen_clear
    GLOBAL      screen_print
    GLOBAL      screen_set_row
    GLOBAL      screen_set_col
    end